package lang
{
import com.felixrum.Log
import com.felixrum.StringTools
import com.greensock.TweenMax

import flash.display.Loader;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.external.ExternalInterface;
	import flash.events.IOErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
import flash.system.Capabilities


public class Localization extends EventDispatcher
	{
		private static var instance;
    	public static const CUSTOM:String = "custom"
    	public static const AUTO:String = "auto"
		public static function getInstance():Localization {
			if (!instance) instance = new Localization();

			return instance;
		}

		public static function get(key: String, replaceKeys:Array=null): String
		{
            return getInstance().get(key, replaceKeys);
			//return ">" + getInstance().get(key, replaceKeys);
		}

		private var defaultLocale:String = "en_US"
		public var locale:String = defaultLocale;
		private var data: Object = {};

		private var languageMap: Object = {
			"en": "en_US",
			"pt": "pt_BR"
		}

		public function getLocale():String{
			return locale
		}

		public function loadLocale(filename):void
		{
            Log.say("Load lang file "+filename, instance)
			var loader:URLLoader = new URLLoader();
			loader.load(new URLRequest(filename));
			loader.addEventListener(Event.COMPLETE, this.localeLoaded);
			loader.addEventListener(IOErrorEvent.IO_ERROR, this.localeLoadingError);
		}

		private function localeLoadingError(error: IOErrorEvent)
		{
			Log.say("Error: Failed to load custom language "+error, instance)
			TweenMax.delayedCall(2, function(){
                if(Polytopia.view) Polytopia.view.alert(error.text, "Failed to load custom language")
			})
			if(locale!=defaultLocale){
                //load fallback
                Log.say("Try default locale: "+defaultLocale, instance)
				setLocale(defaultLocale)
            }else{
				//default locale failed to load
                forceTribeNames()
                dispatchEvent(new Event(Event.COMPLETE));
            }
		}

		private function localeLoaded(event: Event):void
		{
            Log.say("Locale loaded "+locale, instance)
			const data:String = String(event.target.data);
			this.data = JSON.parse(data);
			//set times strings
			StringTools.timeLang.day=get("date.day")
			StringTools.timeLang.days=get("date.days")
			StringTools.timeLang.hour=get("date.hour")
			StringTools.timeLang.hours=get("date.hours")
			StringTools.timeLang.minute=get("date.minute")
			StringTools.timeLang.minutes=get("date.minutes")
			StringTools.timeLang.second=get("date.second")
			StringTools.timeLang.seconds=get("date.seconds")

			forceTribeNames()
            dispatchEvent(new Event(Event.COMPLETE));
		}

		private function forceTribeNames():void {
			//Force default names of all tribes and Nature
            data["tribes.nature"]= "Nature"
			data["tribes.xin-xi"]= "Xin-xi"
            data["tribes.imperius"]= "Imperius"
            data["tribes.bardur"]="Bardur"
            data["tribes.oumaji"]="Oumaji"
            data["tribes.kickoo"]="Kickoo"
            data["tribes.hoodrick"]="Hoodrick"
            data["tribes.luxidoor"]="Luxidoor"
            data["tribes.vengir"]="Vengir"
            data["tribes.zebasi"]="Zebasi"
            data["tribes.aimo"]="Ai-Mo"
            data["tribes.aquarion"]="Aquarion"
            data["tribes.quetzali"]="Quetzali"
            data["tribes.elyrion"]="∑∫ỹriȱŋ"
            data["tribes.yadakk"]="Yădakk"
            data["tribes.polaris"]="Polaris"

        }

		public function injectData(input:String){
            this.data = JSON.parse(input);
		}

		public function setLocaleFromOS():void {

            //autodetect language
            var languageSettings:Array = Capabilities.languages;
            var lang:String = ""
            if (languageSettings.length)
                lang = languageSettings[0].toString().toLowerCase()

            Log.say("Auto find the language of the OS: " + lang, Localization)
            var language:String = String(lang).split("-")[0];
            if (languageMap.hasOwnProperty(language)) {
            	this.setLocale(languageMap[language])
        	}else{
        		this.setLocale(defaultLocale)
    		}

		}

		public function setLocale(locale:String):void
		{
			this.locale = locale;
			this.loadLocale("lang/" + this.locale + ".json");
		}

        private function findRecursive(data, keys)
		{
            if (!keys.length) return null;

            const currentKey = keys.pop();
            if (keys.length == 0) {
                return data[currentKey]
            }
            if (data.hasOwnProperty(currentKey)) {
                return findRecursive(data[currentKey], keys)
            }

            return null;
        }

		private const _space:RegExp = / /g;
		private function sanitizeKey(key:String):String
		{
			if(!key || key==null) return ""
			return key.toLowerCase().replace(_space, "");
		}

		private function resolveKey(key:String):String
		{
            if (data.hasOwnProperty(key))
				return data[key];

            const keys = key.split(".").reverse();
			return findRecursive(data, keys)
		}

		private function replace(string:String, keys:Array):String
		{
			var s = string;
			for (var i:int in keys) {
				const key = "{" + i + "}";
				const value = keys[i];
                s = s.split(key).join(value);
			}
			return s;
		}

		public function get(key:String, replaceKeys:Array=null):String
		{
			if(key=="") return "-"
            const saneKey = sanitizeKey(key);
			const translation = this.resolveKey(saneKey)
            if (!translation) {
				//Log.say("no value in "+ this.locale +" for key '"+ saneKey +"'", Localization)
				return key;
			}

			if (replaceKeys)
				return replace(translation, replaceKeys)

			return translation;
        }

		public static function hasKey(key:String):Boolean{
            const saneKey = getInstance().sanitizeKey(key);
            return Boolean(getInstance().resolveKey(saneKey))
		}
	}
}
